module HousingMisc
  class ModelReflectionsController < ApplicationController
    include ModelsReflectionsHelper
    include HousingMisc::ApiHelper
    before_action :validate_params
    before_action :internal_host_check

  def check_permission
    return false
  end

=begin
  *Author*  Prabal Partap
  <b>Date</b>5 March, 2019
  <b>Short Description</b> Api to get reflections(associations) of the given models and also the models associated with those models(found in reflections of those models) 
  <b>Endpoints</b> Database Prunning Service
  <b>Request Type</b> GET
  <b>Route </b> /model_reflections
  <b>Authentication Required</b> its an internal API hence no authentication
  <b>Request Format</b>
  {
    "models" : "WhatsappPermission,SmsRequest"
  }
  <b>unsuccesful Response</b>
  {
    "message" : "Bad request",
    "error" : "incorrect model names, don't give spaces after model names"
  }
  <b>succesful Response</b>
  {
    "message": "succesful",
    "error": "",
    {
      "whatsapp_permissions": {
        "subscriptions": {
          "table_name": "subscriptions",
          "macros": [
              "belongs_to"
          ],
          "throughs": [
             "FALSE"
          ],
          "foreign_keys": [
            "number"
          ],
          "primary_keys": [
            "number"
          ],
          "model_name": "Subscription"
        }
      },
      "sms_requests": {
        "statuses": {
          "table_name": "statuses",
          "macros": [
            "belongs_to"
          ],
          "throughs": [
            "FALSE"
          ],
          "foreign_keys": [
            "status_id"
          ],
          "primary_keys": [
            "id"
          ],
          "model_name": "::Status"
        },
        "sms_templates": {
          "table_name": "sms_templates",
          "macros": [
            "belongs_to"
          ],
          "throughs": [
            "FALSE"
          ],
          "foreign_keys": [
            "template_id"
          ],
          "primary_keys": [
            "id"
          ],
          "model_name": "::SmsTemplate"
        }
      },
      "subscriptions": {
        "whatsapp_permissions": {
          "table_name": "whatsapp_permissions",
          "macros": [
            "has_many"
          ],
          "throughs": [
            "FALSE"
          ],
          "foreign_keys": [
            "number"
          ],
          "primary_keys": [
            "number"
          ],
          "model_name": "WhatsappPermission"
        }
      },
      "statuses": {},
      "sms_templates": {
        "gateway_types": {
          "table_name": "gateway_types",
          "macros": [
            "belongs_to"
          ],
          "throughs": [
            "FALSE"
          ],
          "foreign_keys": [
            "gateway_type_id"
          ],
          "primary_keys": [
            "id"
          ],
          "model_name": "GatewayType"
        }
      },
      "gateway_types": {
        "sms_templates": {
          "table_name": "sms_templates",
          "macros": [
            "has_many"
          ],
          "throughs": [
            "FALSE"
          ],
          "foreign_keys": [
            "gateway_type_id"
          ],
          "primary_keys": [
            "id"
          ],
          "model_name": "SmsTemplate"
        }
      }
    }
  }
=end

    def models_reflections_generator
      base_models = filter_params['models'].split(',')
      base_tables = []
      table_name = ""
      base_models.each do |model|
        table_name = model.constantize.table_name rescue "Invalid_Table"
        break if table_name == "Invalid_Table" 
        base_tables.push(table_name)
      end
      if table_name == 'Invalid_Table'
        render json: {message: "incorrect model names, don't give spaces after model names"},status: :bad_request
      else
        tables_reflections_hash = tables_reflections_hash_generator(base_models)
        render json: tables_reflections_hash, status: :ok 
      end
    end

    private
    def filter_params
      params.permit(:models)
    end

    def validate_params
      render json: {message: "model name is mandatory"}, status: :bad_request if params[:models].blank?
    end
  end
end

